module.exports = function (db) {
    var util = require('../util');
    var dao = require('../dao')(db);
    var W = require('when');
    return {
        create: function (req, res) {
            if (util.validateParams(res, {"description": req.body.description})) {
                util.getUserIdByToken(req.params.idUser) //ID = access_token
                    .then(function (userID) {
                        return W.all([dao.User.getByIdFacebook(userID), userID]);
                    })
                    .spread(function (user, userID) {
                        if (!user) util.reject("User not registered with id = " + userID);
                        else return W.all([dao.getById(db.Product, req.body.idProduct), dao.findAllWhereInclude(db.BasketLine, {user: user.id}, [
                            { model: db.Product, as: 'product2'}
                        ]), user]);
                    })
                    .spread(function (product, basketlines, user) {
                        if (product.quantity < req.body.quantity || req.body.quantity <= 0 || parseInt(req.body.quantity) === NaN)util.reject("Invalid quantity");
                        else {
                            if (basketlines.length == 0) {
                                if (req.body.quantity * product.coins > user.coins) util.reject("Insufficient visuals");
                                else return W.all([user.getBasketline({ where: {product: product.id}}), product, user]);
                            } else {
                                var df = W.defer();

                                var total = 0;
                                W.iterate(function (i) {
                                    return i + 1;
                                }, function (i) {
                                    // Stop when i >= length
                                    return i >= basketlines.length;
                                }, function (i) {
                                    total += basketlines[i].quantity * basketlines[i].product2.coins;
                                    if (i === basketlines.length - 1) {
                                        console.log(total)
                                        if (total > user.coins) df.resolve([null, null, null]);
                                        else df.resolve([user.getBasketline({ where: {product: product.id}}), product, user]);
                                    }
                                }, 0).done();
                                return df.promise;
                            }
                        }
                    })
                    .spread(function (basketline, product, user) {
                        if (!basketline && !product && !user)util.reject("Insufficient visuals");
                        else if (basketline[0])return W.all([null, null, basketline[0].updateAttributes({quantity: req.body.quantity, description: req.body.description})]);
                        else return W.all([product, user, dao.create(db.BasketLine, {"quantity": req.body.quantity, description: req.body.description})]);
                    })
                    .spread(function (product, user, basketline) {
                        if (product)return W.all([product.addBasketline(basketline), user.addBasketline(basketline), basketline]);
                        else return W.all([product, user, basketline]);
                    })
                    .spread(function (product, user, basketline) {
                        return basketline;
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        read: function (req, res) {
            console.log(req.params.id);
            util.getUserIdByToken(req.params.idUser) //ID = access_token
                .then(function (userID) {
                    return W.all([dao.User.getByIdFacebook(userID), userID]);
                })
                .spread(function (user, userID) {
                    if (!user) util.reject("User not registered with id = " + userID);
                    else if (req.params.id)return user.getBasketline({ where: {product: req.params.id} });
                    else return dao.findAllWhereInclude(db.BasketLine, {user: user.id}, [
                            { model: db.Product, as: 'product2'}
                        ]);
                })
                .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                .done();
        },
        delete: function (req, res) {
            util.getUserIdByToken(req.params.idUser) //ID = access_token
                .then(function (userID) {
                    return W.all([dao.User.getByIdFacebook(userID), userID]);
                })
                .spread(function (user, userID) {
                    if (!user) util.reject("User not registered with id = " + userID);
                    else return user.getBasketline({ where: {id: req.params.id} });
                })
                .then(function (basketline) {
                    if (basketline.length == 0) util.reject("BasketLine not exists = " + req.params.id);
                    else return basketline[0].destroy();
                })
                .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                .done();
        },
        proceed: function (req, res) {
            util.getUserIdByToken(req.params.idUser) //ID = access_token
                .then(function (userID) {
                    return W.all([dao.User.getByIdFacebook(userID), userID]);
                })
                .spread(function (user, userID) {
                    if (!user) util.reject("User not registered with id = " + userID);
                    else return W.all([user, dao.findAllWhereInclude(db.BasketLine, {user: user.id}, [
                        { model: db.Product, as: 'product2'}
                    ])]);
                })
                .spread(function (user, basketlines) {
                    if (basketlines.length == 0) util.reject("Nothing to buy");
                    else {
                        var df = W.defer();

                        var total = 0;
                        W.iterate(function (i) {
                            return i + 1;
                        }, function (i) {
                            // Stop when i >= length
                            return i >= basketlines.length;
                        }, function (i) {
                            if (basketlines[i].quantity > basketlines[i].product2.quantity)df.resolve([basketlines[i].product2, null, null]); //quantity error
                            total += basketlines[i].quantity * basketlines[i].product2.coins;
                            if (i === basketlines.length - 1) {
                                console.log(total)
                                if (total > user.coins) df.resolve([null, null, null]);
                                else df.resolve([null, user, basketlines]);
                            }
                        }, 0).done();
                        return df.promise;
                    }
                })
                .spread(function (product, user, basketlines) {
                    if (product)util.reject("Quantity error, there are only " + product.quantity + " units of" + product.name);
                    else if (!product && !user && !basketlines)util.reject("Insufficient visuals");
                    else {

                        var df = W.defer();
                        var total = 0;
                        W.iterate(function (i) {
                            return i + 1;
                        }, function (i) {
                            // Stop when i >= length
                            return i >= basketlines.length;
                        }, function (i) {
                            dao.getById(db.Product, basketlines[i].product2.id)
                                .then(function (product) {
                                    var quantityBasket = basketlines[i].quantity
                                    return W.all([user.decrement('coins',( quantityBasket*product.dataValues.coins)),
                                        product.decrement('quantity', quantityBasket),
                                        dao.create(db.SaleLine, {quantity: basketlines[i].quantity, description: basketlines[i].description}),
                                        dao.getById(db.Company, product.dataValues.companyid)])

                                })
                                .spread(function (user2,product, sale, company) {
                                    return W.all([
                                    product.addSaleline(sale),
                                    company.addSaleline(sale),
                                    user.addSaleline(sale),
                                    dao.deleteById(db.BasketLine,basketlines[i].id)]);
                                }).done();

                            if (i === basketlines.length - 1) {
                                df.resolve();
                            }
                        }, 0).done();
                        return df.promise;
                    }
                })
                .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                .done();
        }
    }
}