module.exports = function (db) {
    var util = require('../util');
    var dao = require('../dao')(db);
    var W = require('when');
    return {
        login: function (req, res) {
            var attribs = {"email": req.body.email, "password": req.body.password};
            if (util.validateParams(res, attribs)) {
                dao.Company.getByEmail(attribs.email)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with email: " + attribs.email);
                        else return W.all([company, util.BCRYPTcompare(attribs.password,company.password)]);
                    })
                    .spread(function (company,equals){
                        if (!equals) util.reject("Password is not correct");
                        else req.session.company_id = company.id;
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        logout: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                req.session.destroy();
                util.stdSeqSuccess(res, "Logged out");
            }
        },
        create: function (req, res) {
            console.log("create");
            if(req.body.passwordAdmin!="tfg2014")util.stdErr400(res, "Secret password wrong");
            else {
                var attribs = {"name": req.body.name, "type": req.body.type, "street": req.body.street, "city": req.body.city,
                    "country": req.body.country, "postcode": req.body.postcode, "email": req.body.email, "password": req.body.password};
                if (util.validateParams(res, attribs)) {
                    db.sequelize.transaction(function (t) {
                        W.all([dao.Company.getByEmail(attribs.email, t), util.BCRYPThash(attribs.password)])
                            .spread(function (company, password) {
                                if (company) util.reject("Already exist a company with email: " + attribs.email); //util.reject("Already exist a user with email = " + attribs.email);
                                else {
                                    attribs["password"] = password;
                                    return dao.create(db.Company, attribs, t);
                                }
                            })
                            .then(function (company) {
                                if (company)req.session.company_id = company.id;
                            })
                            .then(util.commit.genFuncLeft(t), util.rollback.genFuncLeft(t))
                            .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                            .done();
                    })
                }
            }
        },
        read: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                var attribs= ["id","name", "type", "street", "city", "country", "postcode","email"];
                dao.getByIdAttributes(db.Company, attribs, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return company;
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        update: function (req, res) {
            console.log("update");
            var attribs = {"name": req.body.name, "type": req.body.type, "street": req.body.street, "city": req.body.city,
                "country": req.body.country, "postcode": req.body.postcode, "email": req.body.email, "oldpassword": req.body.oldpassword};

            if (util.validateParams(res, {"company_id": req.session.company_id}) && util.validateParams(res, attribs)) {
                if (req.session.company_id != req.params.id) util.reject("What are you doing little hacker?");
                else {
                    db.sequelize.transaction(function (t) {

                        dao.getById(db.Company, req.params.id, t)
                            .then(function (company) {
                                if (company)
                                {
                                    if(req.body.password)return W.all([dao.Company.getByEmail(attribs.email, t), company,util.BCRYPTcompare(attribs.oldpassword,company.password),util.BCRYPThash(req.body.password)]);
                                    else return W.all([dao.Company.getByEmail(attribs.email, t), company,util.BCRYPTcompare(attribs.oldpassword,company.password),null]);
                                } //searching for duplicate email
                                else util.reject("Company not registered with id = " + req.session.company_id);
                            })
                            .spread(function (company2, company, equals,password) {
                                if(!equals)util.reject("Wrong old password");
                                else if (company2 == null || company2.id == company.id)
                                {
                                    if(password)attribs['password']=password;
                                    company.updateAttributes(attribs);
                                }
                                else util.reject("Already exist a company with email: " + attribs.email);
                            })
                            .then(util.commit.genFuncLeft(t), util.rollback.genFuncLeft(t))
                            .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                            .done();
                    })
                }
            }
        },
        delete: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.deleteById(db.Company, req.session.company_id)
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        }
    }
};