module.exports = function (db) {
    var util = require('../util');
    var dao = require('../dao')(db);
    var W = require('when');
    var fs = require('fs');
    var path = require('path');
    var gm = require('gm');
    return {
        create: function (req, res) {
            if (!req.files) util.stdErr400(res, "File undefined");
            else {
                var attribs = {"name": req.body.name, "ext": path.extname(req.files.file.name).toLowerCase()};
                if (util.validateParams(res, {"company_id": req.session.company_id}) && util.validateParams(res, attribs)) {
                    db.sequelize.transaction(function (t) {
                    dao.getById(db.Company, req.session.company_id,t)
                        .then(function (company) {
                            if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                            else return W.all([dao.create(db.Logo, attribs,t), company]);
                        })
                        .spread(function (logo, company) {
                            if (logo) {
                                var tempPath = req.files.file.path;
                                var targetPath = path.resolve('./public/uploads/logo/' + logo.id + path.extname(req.files.file.name).toLowerCase());
                                if (path.extname(req.files.file.name).toLowerCase() === '.jpg' || path.extname(req.files.file.name).toLowerCase() === '.png')
                                    return W.all([fs.rename(tempPath, targetPath), company.addImage(logo), targetPath,util.commit(t)]);
                                else return W.all([fs.unlink(tempPath), null, null,util.rollback("Only .png and .jpg files are allowed!")]);
                            }
                        })
                        .spread(function (fs, logo, targetPath,transaction) {
                            if (targetPath && logo) {
                                gm(targetPath).contrast("+3").monochrome().write('./public/uploads/logo/' + logo.id + "m" + path.extname(req.files.file.name).toLowerCase(), function (err) {
                                    if (!err) console.log('done');
                                    else console.log(err);

                                });
                                return logo;
                            }
                        })
                        .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                        .done();
                    })
                }
            }
        },
        read: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.id);
                        else if (req.params.id) return company.getImages({ where: {id: req.params.id} });
                        else return company.getImages();
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        delete: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                db.sequelize.transaction(function (t) {
                dao.getById(db.Company, req.session.company_id,t)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return W.all([company.getImages({ where: {id: req.params.id} }),company]);
                    })
                    .spread(function (logo,company) {
                        if (logo.length == 0) util.reject("Logo not exists = " + req.params.id);
                        else {
                            return company.removeImage(logo[0]);
                            //Don't destroy the logo for historial matches
                            //fs.unlink('./public/uploads/logo/' + logo[0].dataValues.id + logo[0].dataValues.ext);
                            //return logo[0].destroy();
                        }
                    })
                    .then(util.commit.genFuncLeft(t), util.rollback.genFuncLeft(t))
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
                })
            }
        }
    }
};