module.exports = function (db) {
    var util = require('../util');
    var dao = require('../dao')(db);
    var W = require('when');
    var request = require('request');
    var fs = require('fs');
    return {
        create: function (req, res) {
            util.getUserIdByToken(req.body.access_token) //ID = access_token
                .then(function (userID) {
                    console.log("user " + userID);
                    return W.all([dao.User.getByIdFacebook(userID), userID]);
                })
                .spread(function (user, userID) {
                    if (!user) util.reject("User not registered with id = " + userID);
                    else return user;
                })
                .then(function (user) {
                    return W.all([user.getImages({ where: {id_photo: req.body.id_photo} }), user]);
                })
                .spread(function (images, user) {
                    if (images[0]) util.reject("Image uploaded");
                    else if (req.body.logo)
                    {
                        var visuals = Math.round((user.friends / 500 + req.body.likes / 36 + req.body.comments / 36) * 12.5);

                        var attribs = {"id_photo": req.body.id_photo,
                            "source": req.body.source,
                            "date": req.body.date,
                            "comments": req.body.comments,
                            "likes": req.body.likes,
                            "value": Math.round(visuals * 0.8)};
                        return W.all([dao.create(db.Photo, attribs), dao.getByWhere(db.Logo, {name: req.body.logo}), user]);
                    }
                    else return W.all([util.GMuploadFile(req.body.source, './public/tmp/' + req.body.id_photo + '.jpg'), dao.findAll(db.Logo, ['companyid IS NOT NULL']), user]);
                })
                .spread(function (image, logos, user) {
                    console.log(1);
                    if(req.body.logo)
                    {
                        if (logos.length==0)return W.all([image, dao.create(db.Logo, {name: req.body.logo}), user, image.dataValues.value]);
                        else return W.all([image, logos[0], user, image.dataValues.value]);
                    }
                    else if (logos.length === 0) {
                        util.reject("There are no logos in our database, try later");
                    }
                    else {
                        var length = logos.length;
                        var df = W.defer();
                        var logo = 0;
                        var min = 1;
                        var count = 0;
                        W.iterate(function (i) {
                            return i + 1;
                        }, function (i) {
                            // Stop when i >= length
                            return i >= length;
                        }, function (i) {

                            var path = './public/uploads/logo/';
                            var name = logos[i].dataValues.id + "m" + logos[i].dataValues.ext;

                            util.GMsize(path + name)
                                .then(function (size) {
                                    return W.all([size, util.GMsize('./public/tmp/' + req.body.id_photo + '.jpg')]);
                                })
                                .spread(function (sizeLogo, sizePhoto) {
                                    var diffWidth = sizeLogo.width / sizePhoto.width;
                                    var diffHeight = sizeLogo.height / sizePhoto.height;
                                    var height = null;
                                    var width = null;
                                    if (diffHeight < diffWidth)height = sizePhoto.height * diffWidth;
                                    else height = sizeLogo.height;
                                    return util.GMcompress('./public/tmp/' + req.body.id_photo + '.jpg', './public/tmp/r' + i + req.body.id_photo + '.jpg', width, height, sizeLogo.width, sizeLogo.height);
                                })
                                .then(function (compressed) {
                                    if (compressed)return util.GMcompare('./public/tmp/r' + i + req.body.id_photo + '.jpg', path + name)
                                })
                                .then(function (diff) {
                                    //fs.unlink('./public/tmp/r' +i+ req.body.id_photo + '.jpg');
                                    console.log("DIFF" + diff);
                                    count++;
                                    if (diff < min) {
                                        logo = i;
                                        min = diff;
                                    }
                                    if (count === logos.length) {
                                        if (min < 0.40) {
                                            //console.log("found it: " + min)

                                            var visuals = Math.round((user.friends / 500 + req.body.likes / 36 + req.body.comments / 36) * 12.5);

                                            var attribs = {"id_photo": req.body.id_photo,
                                                "source": req.body.source,
                                                "date": req.body.date,
                                                "comments": req.body.comments,
                                                "likes": req.body.likes,
                                                "value": Math.round(visuals * (1 - min))};
                                            df.resolve([dao.create(db.Photo, attribs), logos[logo], user, attribs.value]);
                                        } else df.resolve([null, null, null, null]);
                                    }

                                }).done();
                        }, 0).done()
                    }
                    ;
                    return df.promise;
                })
                .spread(function (image, logo, user, value) {
                    if (image) {
                        return W.all([image,logo, user.addImage(image), logo.addImage(image), user.increment('coins', value)]);
                    }
                    else {
                        util.reject("Any logo match with your photo, try later");
                    }
                })
                .spread(function (image,logo, user, logo2, user2) {
                    return [image, logo.name];
                })
                .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                .done();
        },
        read: function (req, res) {
            util.getUserIdByToken(req.params.id) //ID = access_token
                .then(function (userID) {
                    console.log("user " + userID);
                    return W.all([dao.User.getByIdFacebook(userID), userID]);
                })
                .spread(function (user, userID) {
                    if (!user) util.reject("User not registered with id = " + userID);
                    else return user;
                })
                .then(function (user) {
                    return dao.findAllWhereInclude(db.Photo, {user: user.id}, [
                        { model: db.Logo, as: 'image'}
                    ]);
                })
                .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                .done();
        },
        readCompany: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return company.getImages();
                    })
                    .then(function (logos) {
                        console.log(logos.length);
                        var ids = [];

                        for (var i = 0; i < logos.length; i++)
                            ids.push(logos[i].id);

                        return dao.findAllWhereInclude(db.Photo, {logo: {in: ids}}, [
                            { model: db.Logo, as: 'image'},
                            {model: db.User, as: 'user2'}
                        ]);
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        readStatistics: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return company.getImages();
                    })
                    .then(function (logos) {
                        console.log(logos.length);
                        var ids = [];

                        for (var i = 0; i < logos.length; i++)
                            ids.push(logos[i].id);

                        return dao.findAllWhereInclude(db.Photo, {logo: {in: ids}}, [
                            { model: db.Logo, as: 'image'},
                            {model: db.User, as: 'user2'}
                        ]);
                    })
                    .then(function (photos) {
                        var df = W.defer();
                        if (photos.length == 0)return df.resolve();
                        else {
                            var length = photos.length;
                            var collection = new Object();
                            W.iterate(function (i) {
                                    return i + 1;
                                }
                                , function (i) {
                                    return i >= length;
                                } // Stop when i >= length
                                , function (i) {
                                    if(photos[i].dataValues.user2) {
                                        var logo = collection[photos[i].dataValues.image.id];
                                        var age = util.calculateAge(photos[i].dataValues.user2.birthday);
                                        var male = 0,
                                            female = 0,
                                            range1 = 0, range2 = 0, range3 = 0, range4 = 0;
                                        if (age < 18)range1 = 1;
                                        else if (age < 25)range2 = 1;
                                        else if (age < 31)range3 = 1;
                                        else range4 = 1;

                                        if (photos[i].dataValues.user2.gender == "male")male = 1;
                                        else female = 1;

                                        if (logo) //If logo exists
                                        {
                                            logo.likes += photos[i].dataValues.likes;
                                            logo.comments += photos[i].dataValues.comments;
                                            logo.value += photos[i].dataValues.value;
                                            logo.male += male;
                                            logo.female += female;
                                            logo.total += 1;
                                            logo.range1 += range1;
                                            logo.range2 += range2;
                                            logo.range3 += range3;
                                            logo.range4 += range4;
                                        }
                                        else {
                                            collection[photos[i].dataValues.image.id] =
                                            {"id": photos[i].dataValues.image.id,
                                                "ext": photos[i].dataValues.image.ext,
                                                "name": photos[i].dataValues.image.name,
                                                "likes": photos[i].dataValues.likes,
                                                "comments": photos[i].dataValues.comments,
                                                "value": photos[i].dataValues.value,
                                                "male": male,
                                                "female": female,
                                                "total": 1,
                                                "range1": range1,
                                                "range2": range2,
                                                "range3": range3,
                                                "range4": range4
                                            };
                                        }
                                        if (i === length - 1) {
                                            console.log(collection);
                                            df.resolve(collection);
                                        }
                                    }
                                }, 0).done();
                            return df.promise;
                        }
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        }
    }
        ;
}