var W = require('when');
var request = require('request');
var fs = require('fs');
var gm = require('gm');

exports.calculateAge = function (birthday) {
    var splitbirthday = birthday.split("/");
    var birthDay = splitbirthday[1];
    var birthMonth = splitbirthday[0];
    var birthYear = splitbirthday[2];
    var todayDate = new Date();
    var todayYear = todayDate.getFullYear();
    var todayMonth = todayDate.getMonth();
    var todayDay = todayDate.getDate();
    var age = todayYear - birthYear;

    if (todayMonth < birthMonth - 1)age--;
    if (birthMonth - 1 == todayMonth && todayDay < birthDay) age--;

    return age;
}

exports.BCRYPThash = function (password) {
    var df = W.defer();
    var bcrypt = require('bcrypt');
    bcrypt.genSalt(10, function (err, salt) {
        bcrypt.hash(password, salt, function (err, hash) {
            df.resolve(hash);
        });
    });
    return df.promise;
}

exports.BCRYPTcompare = function (password, password2) {
    var df = W.defer();
    var bcrypt = require('bcrypt');
    bcrypt.compare(password, password2, function (err, res) {
        df.resolve(res);
    });
    return df.promise;
}

exports.GMuploadFile = function (url, path) {
    var df = W.defer();
    var w = fs.createWriteStream(path);
    request(url).pipe(w);
    w.on('finish', function () {
        df.resolve();
    });
    return df.promise;
}

exports.GMsize = function (path) {
    var df = W.defer();
    gm(path).size(function (err, value) { //Size logo
        if (!err)df.resolve(value);
        else df.reject(err);
    });
    return df.promise;
}

exports.GMcompressLogo = function (path, path2) {
    var df = W.defer();
    gm(path).contrast("-20").monochrome().write(path2, function (err) {
        if (!err) df.resolve(true);
        else df.resolve(false);
    });
    return df.promise;
}

exports.GMcompress = function (path, path2, width, height, widthLogo, heightLogo) {
    var df = W.defer();
    gm(path).resize(width, height)
        .gravity('Center')
        .resize(widthLogo, heightLogo + "<")
        .crop(widthLogo, heightLogo, 0, 0)
        .contrast("+3")
        .monochrome()
        .write(path2, function (err) {
            if (!err)df.resolve(true);
            else df.reject(err)
        });
    return df.promise;
}

exports.GMcompare = function (path, path2) {
    var df = W.defer();
    gm(path).color(function(err, value){
        if(value===1){
            df.resolve(1); //when an image is fully white or black
        }
        else
        {
            gm.compare(path, path2, function (err, isEqual, equality, raw) {
                if (!err) {
                    console.log("base equality"+equality)
                    if(equality>0.5)equality=1-equality;
                    console.log("Estandard equality", equality);

                    equality*=2;
                    console.log("0-1 equality"+equality);
                    df.resolve(equality);
                }
                else df.reject(err)
            });
        }
    })

    return df.promise;
}

exports.stdSeqSuccess = function (res, obj) {
    res.setHeader('Content-Type', 'application-json');
    res.send(200, obj || {});
};

exports.stdErr500 = function (res, message) {
    res.send(500, {error: message});
};

exports.stdErr400 = function (res, message) {
    res.send(400, {error: message});
};

exports.stdSeqError = function (res, err) {
    res.setHeader('Content-Type', 'application-json');
    exports.stdErr500(res, err.message);
};

exports.reject = function (reason) {
    throw new Error(reason);
};

exports.commit = function (t) {
    var df = W.defer();
    t.commit(t)
        .success(df.resolve)
        .error(df.reject);
    return df.promise;
};

exports.rollback = function (t, err) {
    var df = W.defer();
    t.rollback(t)
        .success(function () {
            df.reject(err)
        })
        .error(function () {
            df.reject(err)
        });
    return df.promise;
};

exports.gentops = function (t) {
    var opts = {};
    if (t) opts.transaction = t;
    return opts;
};

exports.addTrans = function (t, obj) {
    if (!t) return obj;
    else {
        obj.transaction = t;
        return obj;
    }
};

exports.validateParams = function (res, list) {
    for (var key in list)
        if (!list[key]) { //if someone is null
            if (key === "user_id") exports.stdErr400(res, "User not logged");
            else if (key === "company_id")exports.stdErr400(res, "Company not logged");
            else exports.stdErr400(res, key + " undefined");
            return false;
        }
    return true;
};

exports.getUserIdByToken = function (access_token) {
    var df = W.defer();
    var url = "https://graph.facebook.com/me?access_token=" + access_token;
    var request = require('request');
    var r = request(url, function (error, response, body) {
        if (!error) df.resolve(JSON.parse(body).id);
        else df.reject;
    })
    return df.promise;
};